/* siFAX.C
(c) 2002, Andrej Barkhatov (abarkhatov@yahoo.com)
These programs are provided "as is", without any express or implied warranty.
The program is freeware. It is freely distributed subject to the restriction that
this Copyright be included with any distribution
(so that we can continue to disclaim responsibility for any problems that may arise from it).

Based on MGETTY source code.

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdarg.h>
#include <limits.h>
#include <math.h>
#include <float.h>
#include <time.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef _WIN32

#include <sys/times.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sys/termios.h>
#include <unistd.h>

#else

#include <process.h>
#include <io.h>
#include <windows.h>

#endif

int nxtoptind = 1;
char *nxtoptarg;
char *usage =
"Usage:\n%s [option] <fax_name>\n\
Options:\n\
\t-c <num>   COM port number\n\
\t-d <num>   Modem device number (0 = USR, 1 = Rockwell)\n\
\t-m <name>  Multipage (last argument is a mask as '<name>###.g3')\n\
\t-w         Wait and receive\n";

//#ifdef DEBUG
#define PUTS(s) puts(s)
#define PUTCHAR(c) putchar(c)
/*
#else
#define PUTS(s) ()
#define PUTCHAR(c) ()
#endif
*/

#define XLARGEBUFF	1024
#ifndef MAX_PATH
#define MAX_PATH 256
#endif

#define TIME_WAIT 50
#define TIME_DTMF 5
#define TIME_REC 10
#define TIME_POLL 50
#define TIME_CMD 10*TIME_POLL
#define TIME_ATA 3000

#ifndef min
#define min(a,b) ((a)<(b)?(a):(b))
#endif

typedef enum cchar_t {
	NUL, SOH, STX, ETX, EOT, ENQ, ACK, BEL, BS,  HT, LF,
	VT,  FF,  CR,  SO,  SI,  DLE, XON, DC2, XOFF, DC4,
	NAK, SYN, ETB, CAN, EM,  SUB, ESC, FS,  GS, RS,  US
}
cchar_t;

typedef enum ppm_t {
	PPM_MPS, PPM_EOM, PPM_EOP
}
ppm_t;

typedef enum tty_t {
	TTY_ORIGINAL, TTY_COMMAND, TTY_VOICE, TTY_DROPDTR
}
tty_t;

#define MODEL_NUM 2
typedef enum model_t {
	MODEL_USR, MODEL_ROCKW
}
model_t;

#define DLE_ETX "\020\003"
#define CAN_STR "\030"

typedef struct mdm_t {
	int fd;
#ifdef _WIN32
	HANDLE hdl;
#endif
	int model;
	int f_eop;
	int pos;
	int count;
	char buf[XLARGEBUFF + 1];
}
mdm_t;

typedef enum atcmd_t {
	AT_Z, AT_ECHO, AT_INIT,
	AT_F1, AT_F2, AT_F3, AT_F4,
	AT_V1, AT_V2, AT_V3, AT_V4,
	AT_FDT, AT_SETD, AT_D, AT_FDR, AT_SETA, AT_A, AT_VA, AT_VTX, AT_VRX
}
atcmd_t;

char *at_str[][MODEL_NUM] = {
	{"z"								},
	{"e0q0v1" 							},
	{"&f1s0=0x4", 						"&f&k3s0=0&w"},
	{"+fclass=2.0",						"+fclass=2"},
	{"+fip;+fnr=1,1,1,0;+flo=2;+fbo=1",	"+fbor=1"},
	{"+fcc=1,5,0,2,0,0,0,0;+fcr=1",		"+fdcc=1,5,0,2,0,0,0,0;+fcr=1"},
	{""									},
	{"#cls=8"							},
	{"#bdr=8#vsm=128,8000#vsd=1#vss=3", "#bdr=16#vsr=7200#vbs=4#vsd=1"},
	{"#vsp=20#vra=70#vrn=100"			},
	{""									},
	{"+fdt" 							},
	{"x3s0=0s6=1",						"x3s0=0s6=1"},
	{"d1" 								},
	{"+fdr"								},
	{"x4s0=2s6=90s7=90",				"+fbor=0;s0=2"},
	{"a"								},
	{"#vls=0a"							},
	{"#vtx"								},
	{"#vrx"								},
};
#define CMD(m, i) (at_str[i][(m)->model] ? at_str[i][(m)->model] : at_str[i][0])

void delay(int t)
{
#ifndef _WIN32
	usleep(t * 1000);
#else
	Sleep(t);
#endif
}

int mdm_open(mdm_t *mdm, char *name)
{

	PUTS("open");

#ifndef _WIN32
	if ((mdm->fd = open(name, O_RDWR | O_NOCTTY | O_NDELAY)) == -1)
		return 0;
#else
	{
		DWORD dw = 0;
		HANDLE hdl;

		if ((mdm->hdl = hdl = CreateFile(name, GENERIC_READ | GENERIC_WRITE, 0,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE) {
			printf("COM mdm error #%d\n", GetLastError());
			return 0;
		}
		if ((mdm->fd = _open_osfhandle((int) hdl, 0)) == -1)
			return 0;
		if (!PurgeComm(hdl, PURGE_RXABORT|PURGE_TXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR))
			return 0;
		if (!SetupComm(hdl, XLARGEBUFF, XLARGEBUFF))
			return 0;
		if (!ClearCommError(hdl, &dw, NULL))
			return 0;
	}
#endif
	return 1;
}

int mdm_mode(mdm_t *mdm, model_t mode)
{
	static int tio_saved;
#ifndef _WIN32
	static struct termios tio_save;
	struct termios tmp;
	PUTS("mode");

	if (!tio_saved) {
		if (tcgetattr(mdm->fd, &tio_save))
			return 0;
		if (tcflush(mdm->fd, TCIOFLUSH))
			return 0;
		tio_saved = 1;
	}
	memset(&tmp, 0, sizeof(struct termios));
	cfmakeraw(&tmp);
	tmp.c_cflag |= CS8 | CREAD | CLOCAL;
	tmp.c_iflag |= IGNBRK | IGNPAR;
	tmp.c_cc[VMIN] = 1;
	tmp.c_cc[VTIME] = 0;
	if (mdm->fc & FC_CTS)
		tmp.c_cflag |= CRTSCTS;
	if (mdm->fc & FC_XON) {
		tmp.c_iflag &= ~IXANY;
		tmp.c_iflag |= IXON | IXOFF;
		tmp.c_cc[VSTOP] = XOFF;
		tmp.c_cc[VSTART] = XON;
	}
	switch (mode) {
	case TTY_COMMAND:
		cfsetospeed(&tmp, B19200);
		cfsetispeed(&tmp, B19200);
		break;
	case TTY_DROPDTR:
		cfsetospeed(&tmp, B0);
		break;
	case TTY_ORIGINAL:
		if (tio_saved)
			memcpy(&tmp, &tio_save, sizeof(struct termios));
		break ;
	}
	if (tcsetattr(mdm->fd, TCSANOW, &tmp))
		return 0;
	tcflow(mdm->fd, TCOON);
#else
	static DCB oldt;
	static COMMTIMEOUTS oldcto;
	DWORD dw = 0;
	DCB t;
	COMMTIMEOUTS cto;
	HANDLE hdl = mdm->hdl;
	PUTS("mode");

	memset(&cto, 0, sizeof(COMMTIMEOUTS));
#ifdef TIMEOUT
	cto.ReadIntervalTimeout = 3;
	cto.ReadTotalTimeoutMultiplier = 3;
	cto.ReadTotalTimeoutConstant = 100;
	cto.WriteTotalTimeoutMultiplier = 3;
	cto.WriteTotalTimeoutConstant = 100;
#endif
	memset(&t, 0, sizeof(DCB));
	if (!GetCommState(hdl, &t))
		return 0;
	t.DCBlength = sizeof(DCB);
	t.BaudRate = CBR_19200;
	t.ByteSize = 8;
	t.Parity  = NOPARITY;
	t.StopBits = ONESTOPBIT;
	t.fDtrControl = DTR_CONTROL_HANDSHAKE;
	t.fRtsControl = RTS_CONTROL_HANDSHAKE;
	t.fOutxCtsFlow = 1;
	t.fOutxDsrFlow = 1;
	t.fInX = 0;
	t.fOutX = 0;
	switch (mode) {
	case TTY_DROPDTR:
		t.BaudRate = 0;
		break;
	case TTY_ORIGINAL:
		memcpy(&t, &oldt, sizeof(DCB));
		memcpy(&cto, &oldcto, sizeof(COMMTIMEOUTS));
		break;
	case TTY_VOICE:
		t.BaudRate = CBR_38400;
		break;
	}
	if (!tio_saved) {
		tio_saved = 1;
		if (!GetCommState(hdl, &oldt))
			return 0;
		if (!GetCommTimeouts(hdl, &oldcto))
			return 0;
		if (!SetCommTimeouts(hdl, &cto))
			return 0;
	}
	if (!FlushFileBuffers(hdl))
		return 0;
	if (!SetCommState(hdl, &t))
		return 0;
	if (!ClearCommError(hdl, &dw, NULL))
		return 0;
#endif
	return 1;
}

void mdm_close(mdm_t *mdm)
{
	PUTS("close");

	mdm_mode(mdm, TTY_ORIGINAL);
	close(mdm->fd);
}

int mdm_select(mdm_t *mdm, int msecs)
{
#ifndef _WIN32
	fd_set readfds;
	struct timeval timeout, *tptr;
	int slct;

	FD_ZERO(&readfds);
	FD_SET(mdm->fd, &readfds);
	if (msecs >= 0) {
		timeout.tv_sec = msecs / 1000;
		timeout.tv_usec = (msecs % 1000) * 1000;
		tptr = &timeout;
	}
	else
		tptr = NULL;
	slct = select(FD_SETSIZE, &readfds, NULL, NULL, tptr);
	return slct;
#else
	DWORD dw = 0;
	COMSTAT stat;
	HANDLE hdl = mdm->hdl;

	if (msecs < 0) {
		if (!FlushFileBuffers(hdl))
			return 0;
		if (!PurgeComm(hdl, PURGE_RXCLEAR))
			return 0;
		if (!ClearCommError(hdl, &dw, &stat))
			return 0;
		if (!SetCommMask(hdl, EV_RING))
			return 0;

		PUTS("waiting..");
		if (!WaitCommEvent(hdl, &dw, NULL))
			return 0;
		msecs *= -1;
	}
	memset(&stat, 0, sizeof(COMSTAT));
	if (msecs)
		Sleep(msecs);
	if (!ClearCommError(hdl, &dw, &stat))
		return 0;
	return stat.cbInQue;
#endif
}

int mdm_write_raw(int fd, char *buf, int n)
{
	int m;

	while (n > 0) {
		m = write(fd, buf, n);
		if (m > 0) {
			buf += m;
			n -= m;
		}
	}
	return n == 0;
}

int mdm_read_byte(mdm_t *mdm, char *b)
{
	int ret;

	*b = 0;
	if (mdm->pos >= mdm->count) {
		if (!(ret = mdm_select(mdm, TIME_POLL)))
			return 0;
		if ((ret = read(mdm->fd, mdm->buf, min(ret, XLARGEBUFF))) <= 0)
			return 0;
		mdm->count = ret;
		mdm->pos = 0;
	}
	*b = mdm->buf[mdm->pos++];
	return 1;
}

int mdm_read_ln(mdm_t *mdm, char *buf)
{
	char b;
	int n = 0;

	memset(buf, 0, XLARGEBUFF);
	do {
		if (!mdm_read_byte(mdm, &b))
			break;
		if (b != LF && b != CR)
			buf[n++] = b;
	}
	while (!n || (b != LF && n < XLARGEBUFF - 1));

	return n > 0;
}

int mdm_wait(mdm_t *mdm, char *s)
{
	time_t t;
	char buf[XLARGEBUFF], *p;
	PUTS("wait");

	t = time(NULL) + TIME_WAIT;
	while (t > time(NULL)) {
		if (!mdm_read_ln(mdm, buf)) {
			delay(TIME_POLL);
			continue;
		}
		if (strstr(buf, s))
			return 1;
		if ((p = strstr(buf, "+FET:")) != NULL) {

			PUTS(buf);

			if (atoi(p + 5) == 2)
				mdm->f_eop = 1;
		}
		if (strstr(buf, "ERROR") || strstr(buf, "NO") || strstr(buf, "BUSY")
				//|| (strstr(buf, "+FH") && strcmp(buf, "+FH:00")
			) {

			PUTS(buf);

			return 0;
		}
	}
	return 0;
}

void mdm_flush(mdm_t *mdm)
{
	char c;
	PUTS("flush");

	while (mdm_read_byte(mdm, &c));
	return;
}

int mdm_command(mdm_t *mdm, char *cmd, char *answ)
{
	char buf[XLARGEBUFF];
	PUTS("command");

	delay(TIME_CMD);
	strcpy(buf, "at");
	if (cmd && cmd[0])
		strcat(buf, cmd);
	strcat(buf, "\r");

	PUTS(buf);

	if (!mdm_write_raw(mdm->fd, buf, strlen(buf)))
		return 0;
	if (!answ || !answ[0]) {
		mdm_flush(mdm);
		return 1;
	}
	return mdm_wait(mdm, answ);
}

int mdm_sync(mdm_t *mdm)
{
	int i;
	PUTS("sync");

	for (i = 0; i < 3; i++) {
		switch (i) {
		case 0:
			break;
		case 1:
			mdm_mode(mdm, TTY_COMMAND);
			delay(200);
			mdm_mode(mdm, TTY_DROPDTR);
			delay(200);
			mdm_mode(mdm, TTY_COMMAND);
			break;
		case 2:
			mdm_mode(mdm, TTY_COMMAND);
			mdm_write_raw(mdm->fd, CAN_STR, 1);
			mdm_write_raw(mdm->fd, DLE_ETX, 2);
			delay(100);
			mdm_mode(mdm, TTY_COMMAND);
			mdm_write_raw(mdm->fd, CAN_STR, 1);
			mdm_write_raw(mdm->fd, DLE_ETX, 2);
			delay(1500);
			mdm_write_raw(mdm->fd, "+++", 3);
			break ;
		}
		if (mdm_command(mdm, CMD(mdm, AT_ECHO), "OK"))
			break;
	}
	return i < 3;
}

int mdm_send_buf(int fd, char *p, int n)
{
	int i;
	char buf[XLARGEBUFF + 1], c;
	PUTS("send_buf");

	for (i = 0; n > 0 ; n--) {
		c  = *p++;
		if (c == DLE)
			buf[i++] = DLE;
		buf[i++] = c;
		if (n == 1 || i >= XLARGEBUFF) {
			if (!mdm_write_raw(fd, buf, i))
				return 0;
			i = 0;
		}
	}
	return 1;
}

int mdm_fax_send_page(mdm_t *mdm, char *name)
{
	FILE *file = NULL;
	char f = 0, *buf = NULL;
	struct stat fs;
	PUTS("fax_send_page");

	if (stat(name, &fs))
		return 0;
	if ((buf = (char *) malloc(fs.st_size)) == NULL)
		return 0;
	memset(buf, 0, fs.st_size);
	if ((file = fopen(name, "rb")) == NULL)
		goto l;
	if (fread(buf, 1, fs.st_size, file) != fs.st_size)
		goto l;

	PUTS(name);

	if (!mdm_command(mdm, CMD(mdm, AT_FDT), "CONNECT"))
		goto l;

	PUTS("->");

	if (!mdm_send_buf(mdm->fd, buf, fs.st_size))
		goto l;
	f = 1;
l:
	if (buf)
		free(buf);
	fclose(file);

	if (!f)
		PUTS("FAIL");

	return f;
}

int mdm_fax_send_ppm(int fd, ppm_t ppm)
{
	char ppm_char, ppm_buf[2];
	PUTS("fax_send_ppm");

	switch (ppm) {
	case PPM_MPS:
		ppm_char = 0x2c;
		break;
	case PPM_EOM:
		ppm_char = 0x3b;
		break;
	case PPM_EOP:
		ppm_char = 0x2e;
		break;
	default:
		return 0;
	}
	ppm_buf[0] = DLE;
	ppm_buf[1] = ppm_char;

	if (!mdm_write_raw(fd, ppm_buf, 2))
		return 0;

	return 1;
}

int mdm_fax_send(mdm_t *mdm, char *name)
{
	char buf[MAX_PATH];
	int f = 0, last = 0;
	HANDLE h;
	WIN32_FIND_DATA info;
	PUTS("fax_send");

	memset(&info, 0, sizeof(WIN32_FIND_DATA));
	sprintf(buf, "%s*.g3", name);

	PUTS(buf);

	if ((h = FindFirstFile(buf, &info)) == INVALID_HANDLE_VALUE)
		return 0;
	if (!mdm_command(mdm, CMD(mdm, AT_SETD), "OK"))
		goto l;
	mdm_flush(mdm);
	if (!mdm_command(mdm, CMD(mdm, AT_D), "OK"))
		goto l;
	while (!last) {
		if (!mdm_fax_send_page(mdm, info.cFileName))
			goto l;
		if (!FindNextFile(h, &info)) {
			if (!mdm_fax_send_ppm(mdm->fd, PPM_EOP))
				goto l;
			last = 1;
		}
		else {
			if (!mdm_fax_send_ppm(mdm->fd, PPM_MPS))
				goto l;
		}
		if (!mdm_wait(mdm, "OK"))
			return 0;
	}
	if (!mdm_command(mdm, CMD(mdm, AT_INIT), "OK"))
		goto l;
	f = 1;
l:
	FindClose(h);
	return f;
}

int mdm_fax_send_1p(mdm_t *mdm, char *name)
{
	if (!mdm_command(mdm, CMD(mdm, AT_SETD), "OK"))
		return 0;
	mdm_flush(mdm);
	if (!mdm_command(mdm, CMD(mdm, AT_D), "OK"))
		return 0;
	if (!mdm_fax_send_page(mdm, name))
		return 0;
	if (!mdm_fax_send_ppm(mdm->fd, PPM_EOP))
		return 0;
	if (!mdm_wait(mdm, "OK"))
		return 0;
	return 1;
}

int mdm_fax_get_page(mdm_t *mdm, char *name)
{
	time_t t;
	FILE *file;
	char c, f = 0, WasDLE = 0;
	int n = 0;
	PUTS("fax_get_page");

	if ((file = fopen(name, "wb")) == NULL)
		return 0;

	t = time(NULL) + TIME_WAIT;
	while (t >= time(NULL)) {
		if (!mdm_read_byte(mdm, &c)) {

			PUTS("Unable to gbg");

			goto l;
		}
		if (c == 0) {
			fputc(c, file);
			break;
		}
		if (c == DLE) {
			WasDLE = 1;
			break;
		}
	}

	t = time(NULL) + TIME_WAIT;
	while (t >= time(NULL)) {
		if (!mdm_read_byte(mdm, &c)) {

			PUTS("Unable to read byte");

			goto l;
		}
		n++;
		if (!(n & 0xFFF))
			t = time(NULL) + TIME_WAIT;
		if (!WasDLE) {
			if (c == DLE)
				WasDLE = 1;
			else
				fputc(c, file);
		}
		else {
			if (c == DLE)
				fputc(c, file);
			else if (c == SUB)	{
				fputc(DLE, file);
				fputc(DLE, file);
			}
			else if (c == ETX) {

				PUTS("etx");

				break;
			}
			WasDLE = 0;
		}
	}
	f = 1;
l:
	fclose(file);

	if (f) {
		char msg[40];
		sprintf(msg, "%d received", n);

		PUTS(msg);
	}
	else
		PUTS("FAIL");

	return f;
}

int mdm_fax_get_pages(mdm_t *mdm, char *name)
{
	char start_rcv = DC2;
	char buf[MAX_PATH];
	int n = 0;
	PUTS("fax_get_pages");

	mdm->f_eop = 0;
	while (!mdm->f_eop) {
		if (!mdm_command(mdm, CMD(mdm, AT_FDR), "CONNECT"))
			break;

		sprintf(buf, "%s%03d.g3", name, n);

		write(mdm->fd, &start_rcv, 1);

		if (!mdm_fax_get_page(mdm, buf))
			return 0;

		PUTS(buf);

		n++;

		if (!mdm_wait(mdm, "OK"))
			break;
	}
	mdm_command(mdm, CMD(mdm, AT_FDR), "OK");
	return n;
}

int mdm_fax_get(mdm_t *mdm, char *name, int wait)
{
	int n;
	char buf[40];
	PUTS("fax_get");

	mdm_flush(mdm);
	if (!mdm_command(mdm, CMD(mdm, AT_SETA), "OK"))
		return 0;
	if (!mdm_command(mdm, "", "OK"))
		return 0;

	PUTS("Send the fax, please");

	if (wait)
		mdm_select(mdm, - TIME_POLL);
	else
		delay(TIME_ATA);
	if (!mdm_command(mdm, CMD(mdm, AT_A), "OK"))
		return 0;
	if (!(n = mdm_fax_get_pages(mdm, name)))
		PUTS("FAIL (No pages received)");
	else {
		sprintf(buf, "%d received", n);
		PUTS(buf);
	}
	return n > 0;
}

int mdm_fax_init(mdm_t *mdm)
{
	int i;
	PUTS("fax_init");

	for (i = AT_F1; i <= AT_F4; i++)
		if (!mdm_command(mdm, CMD(mdm, i), "OK"))
			return 0;
	return 1;
}

int nextopt(int argc, char **argv, char *args)
{
	char *a, *p;

	if (nxtoptind >= argc || *(a = argv[nxtoptind]) != '-')
		return -1;
	nxtoptind++;
	if (! *(a+1) || ((p = strchr(args, *(a+1))) == 0)) {
		printf( "Unknown option (%s)\n", a);
		return -1;
	}
	if (*(p+1) != ':')
		nxtoptarg = 0;
	else if ( *(a+2) ) nxtoptarg = a+2;
	else if (nxtoptind >= argc) {
		printf("No argument for (%s)\n", a);
		return -1;
	}
	else nxtoptarg = argv[nxtoptind++];
	return *(a+1);
}

int mdm_init(mdm_t *mdm, int model, int com_n)
{
	int ret = 0;
	char buf[40];

	memset(mdm, 0, sizeof(mdm_t));
	mdm->model = model;
#ifdef _WIN32
	sprintf(buf, "\\\\.\\COM%d", com_n);
#else
	sprintf(buf, "/dev/ttyS%d", com_n);
#endif
	if (!mdm_open(mdm, buf))
		return 0;
	if (!mdm_mode(mdm, mdm->model == MODEL_USR ? TTY_COMMAND : TTY_VOICE))
		goto l;
	mdm_command(mdm, CMD(mdm, AT_Z), "OK");
	if (!mdm_sync(mdm))
		goto l;
	if (!mdm_command(mdm, CMD(mdm, AT_INIT), "OK"))
		goto l;
	if (!mdm_fax_init(mdm))
		goto l;
	ret = 1;
l:
	if (!ret) {
		mdm_close(mdm);
		memset(mdm, 0, sizeof(mdm_t));
	}
	return ret;
}

int main(int argc, char *argv[])
{
	int ret = -1, i, com_n = 1, wait = 0, mp = 0, dev = MODEL_USR;
	mdm_t mdm;
	char *templ = NULL, *fax_name = "fax.g3";

	PUTS("siFax - send fax utility (feedback is wellcome: abarkhatov@yahoo.com)");

	if (argc == 1)
		printf(usage);
	else {
		while ((i = nextopt(argc, argv, "c:d:m:w") ) != -1) {
			switch (i) {
			case 'c':
				if ((com_n = atoi(nxtoptarg)) <= 0)
					goto l;
				break;
			case 'd':
				if ((dev = atoi(nxtoptarg)) != MODEL_USR && dev != MODEL_ROCKW)
					goto l;
				break;
			case 'm':
				if (nxtoptarg[0] != '0') {
					templ = nxtoptarg;
					mp = 1;
				}
				break;
			case 'w':
				wait = 1;
				break;
			}
		}
	}
	if (nxtoptarg && nxtoptarg[0]) {
		fax_name = nxtoptarg;
		if (wait && !templ)
			templ = nxtoptarg;
	}
	else if (wait)
		templ = "testing";
	if (!mdm_init(&mdm, dev, com_n))
		goto l;
	if (wait) {
		if (!mdm_fax_get(&mdm, templ, 1))
			goto l;
	}
	else {
		if (mp) {
			if (!mdm_fax_send(&mdm, fax_name))
				goto l;
		}
		else {
		   if (!mdm_fax_send_1p(&mdm, fax_name))
			   goto l;
		}
	}
	ret = 0;
l:
	mdm_close(&mdm);

	if (ret) {
		printf(usage);
		PUTS("FAIL");
	}

	return ret;
}
